<?php

namespace Drupal\openstreetmap;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use GuzzleHttp\Client;

class Overpass {
  /*
   * @var \Drupal\Core\Database\Connection $database
   */
  protected $database;

  /**
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * @var \Drupal\Core\Config\Config|\Drupal\Core\Config\ImmutableConfig
   */
  protected $config;

  /**
   * Constructs a new Overpass object.
   * @param \Drupal\Core\Database\Connection $connection
   */
  public function __construct(Connection $connection, Client $client, ConfigFactory $configFactory) {
    $this->database = $connection;
    $this->httpClient = $client;
    $this->config = $configFactory->get('openstreetmap.settings');
  }

  public function query (string $query) {
    $endpoint = $this->config->get('endpoint');
    if (!$endpoint) throw new \Exception('OpenStreetMap Overpass API endpoint not configured');
    $json = $this->httpClient->request('GET', $endpoint, [
      'query' => [
        'data' => $query
      ]
    ]);
    $contents = $json->getBody()->getContents();
    return $contents;
  }

  public function json (string $query) {
    return json_decode($this->query("[out:json];{$query}out;"));
  }

  public function node (int $node_id) {
    return current($this->json("node({$node_id});")->elements);
  }

  public function way (int $way_id) {
    return current(json_decode($this->query("[out:json];way({$way_id});out geom;"))->elements);
  }

  public function nodesFromQuery($query, $bundle = 'default') {
    $results = json_decode($this->query($query));
    sleep(3);
    if (!$results->elements) return;
    $batch = [
      'title' => 'Adding nodes from query',
      'operations' => []
    ];
    foreach ($results->elements as $element) {
      $batch['operations'][] = [
        ['Drupal\openstreetmap\Entity\OSMNode', 'saveFromElement'],
        [$element, $bundle]
      ];
    }
    batch_set($batch);
  }
}
